#include <ultra64.h>
#include "include/config.h"

#ifndef __GNU_C__
#define inline	/* */
#endif


//==========================================================================//
//= Global Variables =======================================================//
//==========================================================================//


//---=[ Light ]=---//

Lights1  light1 = gdSPDefLights1(
                                 0x10, 0x10, 0x10,      // ambient color
                                 0xaf, 0xaf, 0xaf,      // diffuse color
                                 -127, 0, 127           // dir toward light
                                );



//==========================================================================//
//= Code ===================================================================//
//==========================================================================//




//========================================================================//
//===-[ Static DL's ]-====================================================//
//========================================================================//



//---=[ Initialize RSP state DL ]=---//

Gfx     N64DLinitRSP[] =
{
    gsSPViewport(&vp),
    gsSPClearGeometryMode(G_SHADE | G_SHADING_SMOOTH | G_CULL_BOTH |
			  G_FOG | G_LIGHTING | G_TEXTURE_GEN |
			  G_TEXTURE_GEN_LINEAR | G_LOD ),
    gsSPTexture(0, 0, 0, 0, G_OFF),
    gsSPSetGeometryMode(G_SHADE | G_ZBUFFER),
    gsSPEndDisplayList(),
};



//---=[ Initialize RDP state DL ]=---//

Gfx     N64DLinitRDP[] =
{
    gsDPSetCycleType(G_CYC_1CYCLE),
    gsDPPipelineMode(G_PM_NPRIMITIVE), 
    gsDPSetScissor(G_SC_NON_INTERLACE, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT),
    gsDPSetColorDither(G_CD_ENABLE), 
    gsSPSetLights1(light1),
    gsDPPipeSync(),
    gsSPEndDisplayList(),
};





//---=[ Clear ZBuffer DL ]=---//

Gfx     N64DLclearZbuffer[] =
{
    gsDPSetColorImage( G_IM_FMT_RGBA, G_IM_SIZ_16b, SCREEN_WIDTH,
            OS_K0_TO_PHYSICAL(_zbuf)),
    gsDPPipeSync(),
    gsDPSetCycleType(G_CYC_FILL),
    gsDPSetRenderMode(G_RM_OPA_SURF, G_RM_OPA_SURF),
    gsDPSetFillColor(GPACK_ZDZ(G_MAXFBZ, 0) << 16 | GPACK_ZDZ(G_MAXFBZ, 0)),
    gsDPFillRectangle(0, 0, SCREEN_WIDTH-1, SCREEN_HEIGHT-1),
    gsDPNoOp(), 
    gsDPPipeSync(),

    gsSPEndDisplayList()
};





//---=[ Setup 3d rendering mode ]=---//

Gfx     N64DLset3dRenderMode[] =
{
    gsSPSetGeometryMode( G_SHADE | G_ZBUFFER |  G_LIGHTING),
    gsDPSetCycleType(G_CYC_1CYCLE),
    gsDPPipelineMode(G_PM_NPRIMITIVE), 
    gsDPSetCombineMode(G_CC_SHADE, G_CC_SHADE),
    gsDPSetRenderMode(G_RM_AA_ZB_OPA_SURF, G_RM_AA_ZB_OPA_SURF2),

    gsSPEndDisplayList()
};






//=======================================================================//
//===-[ Dynamics ]-======================================================//
//=======================================================================//






//---=[ Set Viewport ]=---//

void
N64setViewPort(int Xs, int Ys, int Xt, int Yt)
{
    vp.vp.vscale[0]= Xs<<1 ;    vp.vp.vscale[1]= Ys<<1 ;
    vp.vp.vtrans[0]= Xt<<1 ;    vp.vp.vtrans[1]= Yt<<1 ;
}





//---=[ Clear Screen ]=---//

void    N64clearScreen(Gfx **l)
{
 Gfx     *dl ;

    dl = *l ;

    gDPPipeSync(dl++);
    gDPSetCycleType(dl++, G_CYC_FILL);
    gDPSetRenderMode(dl++, G_RM_OPA_SURF, G_RM_OPA_SURF);
    gDPSetFillColor(dl++, ScreenClearColor);
    gDPFillRectangle(dl++, 0, 0, SCREEN_WIDTH-1, SCREEN_HEIGHT-1);
    gDPNoOp(dl++);
    gDPPipeSync(dl++);

    *l = dl ;
}







//---=[ Initialize Main DL ]=---//

void    N64initDL(Gfx** DL_Pos, Gfx* DL_Start, char* rsp_static_addr,
                        Dynamic* generate, u16* CFB, u16 cfbflag,
                        u16 zbufflag)
{
    Gfx     *glistp ;
    int     i;

    glistp      = *DL_Pos ;

    gSPSegment(glistp++, 0, 0x0); 
    gSPSegment(glistp++, STATIC_SEG,  OS_K0_TO_PHYSICAL(rsp_static_addr));
    gSPSegment(glistp++, DYNAMIC_SEG, OS_K0_TO_PHYSICAL(generate));

    gSPDisplayList(glistp++, N64DLinitRSP);
    gSPDisplayList(glistp++, N64DLinitRDP);

    if(cfbflag == YES)
    {
     gDPSetColorImage(glistp++, G_IM_FMT_RGBA, G_IM_SIZ_16b, SCREEN_WIDTH,
                 OS_K0_TO_PHYSICAL(CFB));
     N64clearScreen(&glistp);
    }

    if(zbufflag == YES)
    {
     gSPDisplayList(glistp++, N64DLclearZbuffer);
    }

    gDPSetColorImage(glistp++, G_IM_FMT_RGBA, G_IM_SIZ_16b, SCREEN_WIDTH,
             OS_K0_TO_PHYSICAL(CFB));

    gSPDisplayList(glistp++, N64DLset3dRenderMode);

    gDPSetDepthImage(glistp++, OS_K0_TO_PHYSICAL(_zbuf));

    *DL_Pos     = glistp ;
}





//---=[ End Main DL ]=---//

void    N64terminateDL(Gfx **DL_Pos)
{
    Gfx     *glistp ;

    glistp      = *DL_Pos ;

    gDPFullSync(glistp++); /* Only need this if you want 'accurate' SP done? */
    gSPEndDisplayList(glistp++);

    *DL_Pos     = glistp ;
}





//---=[ Swap Buffers ]=---//

void    N64swapCFB()
{
 static int	first=1;

    if(first)
    {
     osViBlack(FALSE);
     first = 0;
    }

    osViSwapBuffer(Logical_CFB);

    CFB_Count++;
    if (CFB_Count >= NUM_CFB)
     CFB_Count = 0;

    Logical_CFB = _cfb[CFB_Count];

    CFB_NextCount = CFB_Count+1;
    if(CFB_NextCount >= NUM_CFB)
     CFB_NextCount = 0;

    Next_CFB = _cfb[CFB_NextCount] ;
}





//---=[ Wait for VRT ]=---//

void    N64waitVRT()
{
    if(MQ_IS_FULL(&RetraceMessageQ))
      osRecvMesg(&RetraceMessageQ, NULL, OS_MESG_BLOCK);
    osRecvMesg(&RetraceMessageQ, NULL, OS_MESG_BLOCK);
}





//---=[ Wait for RDP ]=---//

void    N64waitRDP(int first)
{
    if(!first)
      osRecvMesg(&RDPDoneMessageQ, NULL, OS_MESG_BLOCK);
}




//---=[ Wait for RSP ]=---//

void    N64waitRSP(int first)
{
    if(!first)
      osRecvMesg(&RSPDoneMessageQ, NULL, OS_MESG_BLOCK);
}




//---=[ Set up rendering task ]=---//

void N64initTask(OSTask* task, Gfx* gdl_end, Gfx* gdl_start)
{
	//=== type ===//
	task->t.type = M_GFXTASK;		// graphics
//	task->t.type = M_AUDTASK;		// audio

	//=== flags ===//
//      task->t.flags = OS_TASK_DP_WAIT;        // non-audio, non-_fifo
        task->t.flags = 0;                      // audio, _fifo

	//=== boot microcode pointer ===//
        task->t.ucode_boot = rspbootTextStart;  // always

	//=== boot microcode length ===//
        task->t.ucode_boot_size=((u32)rspbootTextEnd -
                                (u32)rspbootTextStart); // always

	//=== task microcode pointer ===//
//      task->t.ucode = gspFast3DTextStart;             // graphics 
//      task->t.ucode = gspFast3D_dramTextStart;        // graphics 
        task->t.ucode = gspFast3D_fifoTextStart;        // graphics 
//      task->t.ucode = gspLine3DTextStart;             // graphics
//	task->t.ucode = gspLine3D_dramTextStart;	// graphics
//	task->t.ucode = gspLine3D_fifoTextStart;	// graphics
//      task->t.ucode = aspMainTextStart;               // audio

	//=== task microcode size ===//
        task->t.ucode_size = SP_UCODE_SIZE;             // always

	//=== task microcode data pointer ===//
//      task->t.ucode_data = gspFast3DDataStart;        // graphics
//	task->t.ucode_data = gspFast3D_dramDataStart;	// graphics
        task->t.ucode_data = gspFast3D_fifoDataStart;   // graphics
//      task->t.ucode_data = gspLine3DDataStart;        // graphics
//	task->t.ucode_data = gspLine3D_dramDataStart;	// graphics
//	task->t.ucode_data = gspLine3D_fifoDataStart;	// graphics
//	task->t.ucode_data = aspMainDataStart;		// audio

	//=== task microcode data size ===//
        task->t.ucode_data_size = SP_UCODE_DATA_SIZE;   // always

	//=== DRAM stack (own .o) ===//
        task->t.dram_stack = (u64*)_rspdrambuf;         // graphics
//      task->t.dram_stack = 0;                         // audio

	//=== DRAM stack size ===//
        task->t.dram_stack_size = SP_DRAM_STACK_SIZE8;  // graphics
//	task->t.dram_stack_size = 0;			// audio

	//=== output buffer pointer (own .o) ===//
//      task->t.output_buff = 0;                        // gspFast3D
//	task->t.output_buff = area as big as DP stream	// _dram
        task->t.output_buff = fifobuffer;               // _fifo

	//=== output buffer size (own .o) ===//
//	task->t.output_buff_size = u64* where length will be stored  // _dram
        task->t.output_buff_size = (u64*)fifosize;              // _fifo
//      task->t.output_buff_size = 0;                           // gspFast3D

	//=== data pointer ===//
        task->t.data_ptr = (u64*)gdl_start;                           // gfx
//      task->t.data_ptr = ptr to list created by alAudioFrame        // aud

	//=== data size ===//
        task->t.data_size = ((int) gdl_end - (int) gdl_start);

	//=== yield data ptr ===//
	task->t.yield_data_ptr = 0;

	//=== yield data size ===//
	task->t.yield_data_size = 0;
}



